@@ -112,6 +112,9 @@ class HAttention1D(nn.Module):
 
         q, k, v = map(lambda t: rearrange(t, 'b n (h d) -> (b h) n d', h = h), (q, k, v))
 
+        if exists(mask):
+            mask = repeat(mask, 'b n -> (b h) n', h = h)
+
         # scale
 
         q = q * self.scale
@@ -119,7 +122,7 @@ class HAttention1D(nn.Module):
         # rotary pos emb
 
         if exists(self.pos_emb):
-            freqs = self.pos_emb(torch.arange(n, device = device), cache_key = n)
+            freqs = self.pos_emb(torch.arange(pad_to_len, device = device), cache_key = pad_to_len)
             freqs = rearrange(freqs, 'n d -> () n d')
             q, k, v = map(lambda t: apply_rotary_emb(freqs, t), (q, k, v))
 
@@ -135,7 +138,7 @@ class HAttention1D(nn.Module):
             q, k, v = map(lambda t: rearrange(t, 'b (n r) d -> b n r d', r = 2), (q, k, v))
 
             if exists(mask):
-                mask = rearrange(mask, 'b (n r) -> b n r', r = 2)
+                mask = repeat(mask, 'b (n r) -> b n r', r = 2)
 
             # masked mean for queries and keys, but not values
 
